## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko


## The analysis in Section~6 of the paper

source("bpolr.R")

YL <- factor(rep(1:4, 3))
xL <- rep(c(0,1,1)-1/2, each = 4)
countsL <- c(8, 6, 1, 0,
             10, 0, 1, 0,
             8, 1, 0, 0)

YR <- factor(rep(1:4, 2))
xR <- rep(c(0,1)-1/2, each = 4)
countsR <- c(8, 6, 1, 0,
             18, 1, 1, 0)



##############################################
## Actual data and maximum likelihood
##############################################
## The maximum likelihood fit using the left data set in Table~2
leftML <- bpolr(YL ~ xL, weights = countsL)
## The maximum likelihood fit using the right data set in Table~2
rightML <- bpolr(YR ~ xR, weights = countsR)

## The estimates and the estimated standard error are the same
## irrespectively of the data represenetation
c(leftML$beta, sqrt(diag(vcov(leftML)))[1])
## -1.944498  0.895340
c(rightML$beta, sqrt(diag(vcov(rightML)))[1])
## -1.944498  0.895340



##############################################
## Adjusted data and maximum likelihood
##############################################
## Adjust the counts in both tables by adding 0.5
countsLadj <- countsL + 0.5
countsRadj <- countsR + 0.5

## The estimates and estimated standard errors are not the same
## anymore
leftMLadj <- bpolr(YL ~ xL, weights = countsLadj)
rightMLadj <- bpolr(YR ~ xR, weights = countsRadj)
c(leftMLadj$beta, sqrt(diag(vcov(leftMLadj)))[1])
## -1.0967238  0.6777975
c(rightMLadj$beta, sqrt(diag(vcov(rightMLadj)))[1])
## -1.4851515  0.7405383


## p-values from Wald-test for the hypothesis \beta = 0 versus \beta \ne 0
(pL <- 2*(1- pnorm(abs(leftMLadj$beta/sqrt(diag(vcov(leftMLadj)))[1]))))
## 0.1056475
(pR <- 2*(1 - pnorm(abs(rightMLadj$beta/sqrt(diag(vcov(rightMLadj)))[1]))))
## 0.04490935



##############################################
## Actual data and bias reduction
##############################################
## The reduced-bias fit using the left data set in Table~2
leftBR <- bpolr(YL ~ xL, weights = countsL, method = "BR")
## The reduced-bias fit using the right data set in Table~2
rightBR <- bpolr(YR ~ xR, weights = countsR, method = "BR")

## As for maximum likelihood, the estimates and the estimated standard
## errors are the same irrespectively of the data represenetation
c(leftBR$beta, sqrt(diag(vcov(leftBR)))[1])
        xL         xL
## -1.7608938  0.8503491
c(rightBR$beta, sqrt(diag(vcov(rightBR)))[1])
        xR         xR
## -1.7608938  0.8503491





