## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko

## Before running the current script, the script Section8_9.R
## needs to be run 2 times, with setting the type variable therein to
## "1s" in one run, and to "2s" in the second run.

## The script here assumes that the current directory contains a
## directory named images which contains the results that the script
## Section8_9.R will produce.


rm(list = ls())
source("cImages.R")
source("~bpolr.R")
library(MASS)
library(ordinal)
library(lattice)
library(latticeExtra)


wd <- paste0(getwd(), "/images")
imageNames <- c("compenu_2x6_3_3logit.RData",
                "compenu_2x6_3_3cloglog.RData")

cImages(imageNames, c("fittedML", "fittedBR", "link", "dat", "tots"),
        working.directory = wd)

cImages(imageNames[1], c("ncat"), working.directory = wd)

catInList("fittedML", ".image", 1:length(imageNames), delete = TRUE)
catInList("fittedBR", ".image", 1:length(imageNames), delete = TRUE)
catInList("link", ".image", 1:length(imageNames), delete = TRUE)
catInList("dat", ".image", 1:length(imageNames), delete = TRUE)
catInList("tots", ".image", 1:length(imageNames), delete = TRUE)
cObjects("ncat", ".image", 1, delete = TRUE)


## Shrinkage per category
pdf("shrinkage.pdf", width = 10, height = 8)
q <- ncat - 1
limsx <- c(0, 1)
limsy <- c(0, 1)
low <- -0.1
upp <- 1.1
par(mfrow = c(length(dat), ncat))
for (r in 1:2) {
    where <- make.link(link[[r]])$linkinv
    what <- 1:ncol(dat[[r]])
    for (i in 0:q) {
        plot(fittedML[[r]][i, 1], fittedBR[[r]][i, 1],
             xlab = substitute(hat(pi)[paste(ind)],
             list(ind = i + 1)),
             ylab = substitute(tilde(pi)[paste(ind)],
             list(ind = i + 1)),
             xlim = limsx, ylim = limsy,
             type = "n", cex.lab = 1.3)
        title(paste("Category", i + 1), cex = 2.3)
        abline(0, 1, lty = 2)
        if (i == 0) {
            polygon(x = c(low, low, where(0)), y = c(low, where(0), where(0)),
                    col = "grey90", border = NA)
            polygon(x = c(where(0), upp, upp), y = c(where(0), where(0), upp),
                    col = "grey90", border = NA)
            abline(v = where(0), lty = 2)
            abline(h = where(0), lty = 2)
        }
        else if (i == q) {
            polygon(x = c(low, low, 1 - where(0)), y = c(low, 1 - where(0), 1 - where(0)),
                    col = "grey90", border = NA)
            polygon(x = c(1 - where(0), upp, upp), y = c(1 - where(0), 1 - where(0), upp),
                    col = "grey90", border = NA)
            abline(v = 1 - where(0), lty = 2)
            abline(h = 1 - where(0), lty = 2)
        }
        else {
            polygon(x = c(low, upp, upp),
                    y = c(low, low, upp),
                    col = "grey90", border = NA)
        }
        points(fittedML[[r]][1 + i, what],
               fittedBR[[r]][1 + i, what],
               pch = "+", cex = 0.4)
    }
}
dev.off()

