## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko


## Functions for extracting and concatenating objects from many R images.
##
##
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is",
## licensed under GPL2 or greater.  NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## This code still has several rough edges, some of which are
## indicated by the embedded comments.
##
## The intention is to include a more polished implementation as part
## of the CRAN package "brglm", in due course.


cImages <- function(images.cImages = NULL,
                    objects.cImages = NULL,
                    verbose = TRUE,
                    working.directory = getwd()) {
  wdir <- getwd()
  setwd(working.directory)
  donot.cImages <- c("images.cImages",
                     "objects.cImages",
                     "types.cImages",
                     #
                     "n.cImages",
                     "temp.objs.cImages",
                     "i.cImages",
                     "j.cImages",
                     "donot.cImages")
  n.cImages <- length(images.cImages)
  temp.objs.cImages <- NULL
  for (i.cImages in 1:n.cImages) {
    if (verbose) cat("Retrieving from image", images.cImages[i.cImages], "\n")
    load(images.cImages[i.cImages])
    if (is.null(objects.cImages))
      temp.objs.cImages[[i.cImages]] <- ls()
    else
      temp.objs.cImages[[i.cImages]] <- objects.cImages
    for (j.cImages in temp.objs.cImages[[i.cImages]]) {
      if (j.cImages %in% donot.cImages) next
      assign(paste(j.cImages,".image",i.cImages,sep=""),
             get(j.cImages), envir = .GlobalEnv)
    }
  }
  setwd(wdir)
}

compatibilityCheck <- function(images.cImages = NULL,
                    verbose = TRUE,
                    working.directory = getwd()) {
  wdir <- getwd()
  setwd(working.directory)
  donot.cImages <- c("images.cImages",
                     #
                     "n.cImages",
                     "temp.objs.cImages",
                     "i.cImages",
                     "donot.cImages")
  n.cImages <- length(images.cImages)
  temp.objs.cImages <- NULL
  for (i.cImages in 1:n.cImages) {
    if (verbose) cat("Getting object names from", images.cImages[i.cImages], "\n")
    load(images.cImages[i.cImages])
    temp.objs.cImages[[i.cImages]] <- ls()
  }
  nn <- length(temp.objs.cImages)
  browser()
  if (nn == 1) stop("Only one image name has been supplied")
  mat <- matrix(TRUE, nn, nn)
  ll <- temp.objs.cImages
  for (i in 1:(nn-1))
    for (j in (i+1):nn) {
      if (length(ll[[i]]) != length(ll[[j]])) mat[i, j] <- FALSE
      else mat[i, j] <- all(sort(ll[[i]]) == sort(ll[[j]]))
    }
  if (all(mat)) cat("All images have the same objects")
  else {
    inds <- which(!mat, arr.ind = TRUE)
    for (i in 1:nrow(inds)) cat("Image:", inds[i,1], "contains\n",
                               ll[[inds[i,1]]], "\n",
                               "while image:", inds[i,2], "contains\n",
                               ll[[inds[i,2]]], "\n")
  }
}

rbindObjects <- function(objectName , identifier = "", index = 1, delete = FALSE) {
  objects <- paste(objectName, identifier, index, sep="")
  temp <- NULL
  for (i in objects) {
    temp <- rbind(temp, get(i))
  }
  if (delete) rm(list = objects, envir = .GlobalEnv)
  assign(objectName, temp, envir= .GlobalEnv)
}

cbindObjects <- function(objectName , identifier = "", index = 1, delete = FALSE) {
  objects <- paste(objectName, identifier, index, sep="")
  temp <- NULL
  for (i in objects) {
    temp <- cbind(temp, get(i))
  }
  if (delete) rm(list = objects, envir = .GlobalEnv)
  assign(objectName, temp, envir= .GlobalEnv)
}


catInList <- function(objectName , identifier = "", index = 1, delete = FALSE) {
  objects <- paste(objectName, identifier, index, sep="")
  nofobjects <- length(index)
  temp <- as.list(rep(0, nofobjects))
  for (i in 1:nofobjects) {
    temp[[i]] <- get(objects[i])
  }
  if (delete) rm(list = objects, envir = .GlobalEnv)
  assign(objectName, temp, envir= .GlobalEnv)
}

cObjects <- function(objectName , identifier = "", index = 1, delete = FALSE) {
  objects <- paste(objectName, identifier, index, sep="")
  temp <- NULL
  for (i in objects) {
    temp <- c(temp, get(i))
  }
  if (delete) rm(list = objects, envir = .GlobalEnv)
  assign(objectName, temp, envir= .GlobalEnv)
}
